/****************************************************************************
 Module
   MotorFSM.c
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
#include "ES_Configure.h"
#include "ES_Framework.h"

#include <sys/attribs.h>
#include <stdio.h>
#include <stdint.h>

#include "MotorFSM.h"
#include "SPIService.h"
#include "PIC32PortHAL.h"

/*----------------------------- Module Defines ----------------------------*/
#define IC1_PIN (0b0100) //Pin RB2
#define IC2_PIN (0b0100) //Pin RB9
#define IC_PRESCALE (16)
#define PBCLOCK_FREQ (20000000) //20 MHz

#define MOTOR_PORT (_Port_B)
#define MOTOR1_TOGGLEPIN (_Pin_3)
#define MOTOR1_BITS (BIT3HI)
#define MOTOR2_TOGGLEPIN (_Pin_10)
#define MOTOR2_BITS (BIT10HI)

#define LIMITSWITCH_PORT (_Port_A)
#define LIMITSWITCH_PIN (_Pin_4)

#define ENC_PULSE_PER_REV (3)
#define GEAR_RATIO (50)
#define MIN_CONV (60)

#define FULL_SPEED (PWM_PR*.7)
#define SWEEP_DC (PWM_PR*.3)

#define CW (1)
#define CCW (0)

#define Kp (2) //0.8
#define Ki (0.2) //0.05
#define MOVE2LAUNCH_TIMEOUT (5000) 
#define STOPMOVING_TIMEOUT (1000)
#define SWEEPING_TIMEOUT (2000)

#define PWM_PRESCALE (8)
#define PWM_PR 300 //tau = 147 us

/*---------------------------- Module Functions ---------------------------*/
void ConfigureTimer1WithInterrupt(void); //for control ISR
void SpeedControl_ISR(void);
void PIControlMotor1(void);
void PIControlMotor2(void);

void ConfigureTimer2(void); //for input capture
void ConfigureInputCapture1(void);
void ConfigureInputCapture2(void);
void CalculateSpeed_Motor1(void); //ISR for IC1
void CalculateSpeed_Motor2(void); //ISR for IC2

void ConfigureTimer3(void); //for output compare
void ConfigureOC1_Motor1(void);
void ConfigureOC2_Motor2(void);

void ConfigureTimer4WithInterrupt(void); //for checking if motor stalls
void ConfigureTimer5WithInterrupt(void); //for checking if motor stalls
void Motor1Stall(void);
void Motor2Stall(void);

typedef struct {
    uint8_t Motor1Dir;
    uint32_t Motor1DC;
    uint8_t Motor2Dir;
    uint32_t Motor2DC;
    uint32_t TargetSpeed;
} Execution_t ;

void UpdateMotors(Execution_t ThisCommand);

/*---------------------------- Module Variables ---------------------------*/
static MotorState_t CurrentState;
static uint8_t MyPriority;
static ES_Event_t Command2SendEvent;

static uint8_t LastLimitSwitchValue;

static uint8_t Direction1;
static uint8_t Direction2;
static uint32_t TargetRPM;
static uint8_t SweepingDir;
static uint8_t SweepCount = 1;

union {
    struct {
        uint16_t CapturedTime1;
        uint16_t RolloverCount1;
    } MyWord;
    uint32_t CurrentTime1;
} Time1;
uint32_t LastTime1;
static uint32_t PeriodTicks1;
static uint32_t PeriodTime1;
static float CurrentRPM1;

union {
    struct {
        uint16_t CapturedTime2;
        uint16_t RolloverCount2;
    } MyWord;
    uint32_t CurrentTime2;
} Time2;
uint32_t LastTime2;
static uint32_t PeriodTicks2;
static uint32_t PeriodTime2;
static float CurrentRPM2;

Execution_t StopMoving[1] = {0,0,0,0,0};
Execution_t FullSweeping[1] = {0,100,1,100,30};
Execution_t Move2Launch[1] = {0,100,0,100,80};
Execution_t Move2Reload[1] = {1,100,1,100,60};
Execution_t MoveBack2Wall[1] = {1,100,1,100,60};
Execution_t SweepingCW[1] = {0,70,1,70,15};
Execution_t SweepingCCW[1] = {1,70,0,70,15};

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
    InitMotorFSM

 Parameters
     uint8_t : the priority of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, sets up the initial transition and does any
     other required initialization for this state machine
 Notes

 Author
    Kaylie Wong
****************************************************************************/
bool InitMotorFSM(uint8_t Priority)
{
    clrScrn();
    ES_Event_t ThisEvent;

    MyPriority = Priority;

    //Setup Timer 1 for speed control
    ConfigureTimer1WithInterrupt();
    //Timer 4&5 w/ interrupt setup for checking if motor stalls
    ConfigureTimer4WithInterrupt();
    ConfigureTimer5WithInterrupt();

    //Timer 2 and IC setup
    ConfigureTimer2();
    ConfigureInputCapture1();
    ConfigureInputCapture2();

    //Timer 3 and OC setup
    ConfigureTimer3();
    ConfigureOC1_Motor1();
    ConfigureOC2_Motor2();    

    //Configure Limit Switch as digital input
    PortSetup_ConfigureDigitalInputs(LIMITSWITCH_PORT, LIMITSWITCH_PIN); 

    //Set CurrentState to Waiting
    CurrentState = Waiting;

    // post the initial transition event
    ThisEvent.EventType = ES_INIT;
    if (ES_PostToService(MyPriority, ThisEvent) == true)
    {
      return true;
    }
    else
    {
      return false;
    }
}

/****************************************************************************
 Function
    PostMotorFSM

 Parameters
     EF_Event_t ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
    Kaylie Wong
****************************************************************************/
bool PostMotorFSM(ES_Event_t ThisEvent)
{
  return ES_PostToService(MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunMotorFSM

 Parameters
   ES_Event_t : the event to process

 Returns
   ES_Event_t, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
    Kaylie Wong
****************************************************************************/
ES_Event_t RunMotorFSM(ES_Event_t ThisEvent)
{
  ES_Event_t ReturnEvent;
  ReturnEvent.EventType = ES_NO_EVENT; 

  switch (CurrentState)
  {
    case Waiting:        
    {
      switch (ThisEvent.EventType)
      {
          case ES_INIT:
          {
              printf("ES_INIT in MotorFSM\n\r");
              
              // Clear and start timers
              TMR2 = 0;
              T2CONbits.ON = 1;
              TMR3 = 0;
              T3CONbits.ON = 1;
              TMR4 = 0;
              T4CONbits.ON = 1;
              TMR5 = 0;
              T5CONbits.ON = 1;
                    
              //Initialize CurrentTime, LastTime, & RPM for speed calculation
              Time1.CurrentTime1 = 0;
              LastTime1 = 0;
              PeriodTicks1 = 0;
              CurrentRPM1 = 0;
              Time2.CurrentTime2 = 0;
              LastTime2 = 0;
              PeriodTicks2 = 0;
              CurrentRPM2 = 0;
          }
          break;
          
          case ES_NEW_KEY:
          {
              if (ThisEvent.EventParam == '1')
              {
                printf("Starting alignment\n\r");
                UpdateMotors(FullSweeping[0]);
                //Start speed control
                  TMR1 = 0;
                  T1CONbits.ON = 1;
                
                //Set CurrentState to Aligning
                CurrentState = Aligning;
              }
              else if (ThisEvent.EventParam == '3')
              {
                  printf("Move to Launch\n\r");
                  UpdateMotors(Move2Launch[0]);
                  
                  ES_Timer_InitTimer(MOVE2LAUNCH_TIMER, MOVE2LAUNCH_TIMEOUT);
                  
                  //Start speed control
                  TMR1 = 0;
                  T1CONbits.ON = 1;
                  
                  CurrentState = Moving;
              }
              else if (ThisEvent.EventParam == '4')
              {
                  printf ("Move to Reload\n\r");
                  UpdateMotors(Move2Reload[0]);
                  //Start speed control
                  TMR1 = 0;
                  T1CONbits.ON = 1;
                  CurrentState = Moving;
              }
              else if (ThisEvent.EventParam == '0')
              {
                  printf("Stop moving\n\r");
                  UpdateMotors(StopMoving[0]);
                  T1CONbits.ON = 0;
                  //PeriodTicks1 = 0;
                  //CurrentState = Waiting;
              }
          }
          break;
          
          case ES_RECEIVEDCOMMAND:
          {
            if (ThisEvent.EventParam == 0) //Query command
            {
                printf("Received Query Command\n\r");
                
                //Send command saying Waiting
                Command2SendEvent.EventType = ES_SENDCOMMAND;
                PostSPIService(Command2SendEvent);
            }
            else if (ThisEvent.EventParam == 1) //Received Alignment Command
            {
                printf("Received Alignment Command\n\r");
                
                //Start aligning
                UpdateMotors(FullSweeping[0]);
                //Start speed control
                TMR1 = 0;
                T1CONbits.ON = 1;
                
                //Set CurrentState to Aligning
                CurrentState = Aligning;
            }
            else if (ThisEvent.EventParam == 3) //Move to Launch Position command
            {
                printf("Received Move to Launch Position Command\n\r");
                
                //Zero and start Timer 1 for speed control
                TMR1 = 0;
                T1CONbits.ON = 1;
                
                //Move motors forward
                UpdateMotors(Move2Launch[0]);
                
                //Initialize timer
                ES_Timer_InitTimer(MOVE2LAUNCH_TIMER, MOVE2LAUNCH_TIMEOUT);
                
                //Set CurrentState to Moving
                CurrentState = Moving;
            }
            else if (ThisEvent.EventParam == 4) //Move to Reload Position command
            {
                printf("Received Move to Reload Position Command\n\r");
                
                //Zero and start Timer 1 for speed control
                TMR1 = 0;
                T1CONbits.ON = 1;
                
                //Move motors backward
                UpdateMotors(Move2Reload[0]);
                                
                //Set Current State to Moving
                CurrentState = Moving;
            }
            else if (ThisEvent.EventParam == 0xFF)
            {
                //Stop moving motors
                UpdateMotors(StopMoving[0]);
                T1CONbits.ON = 0;
            }
          }
          break;
          
          default:
              ;
      }
    }
    break;
    
    case Aligning:
    {
        switch (ThisEvent.EventType)
        {
            case ES_RECEIVEDCOMMAND:
            {
                if (ThisEvent.EventParam == 3) //Stop aligning, start moving to wall command
                {
                    printf("Stop aligning & move back to wall \n\r");

                    //Move back to wall until limit switch is detected
                    UpdateMotors(Move2Reload[0]);
                    
                    //Zero and start Timer 1 for speed control
                    TMR1 = 0;
                    T1CONbits.ON = 1;
                    
                    //Set CurrentState to DoneAligning
                    CurrentState = DoneAligning;
                }                
                else if (ThisEvent.EventParam == 6)
                {
                    //Stop sweeping
                    UpdateMotors(StopMoving[0]);
                    T1CONbits.ON = 0;
                    
                    printf("Stop Alignment\n\r");
                    
                    //Set CurrentState to Waiting
                    CurrentState = Waiting;
                }
                else if (ThisEvent.EventParam == 0) //Query command
                {
                    printf("Received query command\n\r");

                    //Send still moving message
                    Command2SendEvent.EventType = ES_SENDCOMMAND;
                    //Command2SendEvent.EventParam = 3;
                    PostSPIService(Command2SendEvent);
                }
                else if (ThisEvent.EventParam == 0xFF)
                {
                    //Stop moving motors
                    UpdateMotors(StopMoving[0]);
                    T1CONbits.ON = 0;

                    //Post ES_INIT and put us back into Waiting
                    CurrentState = Waiting;
                    ThisEvent.EventType = ES_INIT;
                    PostMotorFSM(ThisEvent);
                }
            }
            break;
            
            case ES_TIMEOUT: //Timeout for sweeping
            {
                if (SweepCount > 1)
                {
                    //Initialize timer
                    ES_Timer_InitTimer(SWEEPING_TIMER, 2*SWEEPING_TIMEOUT);
                }
                else //SweepCount <=1, first time
                {
                    ES_Timer_InitTimer(SWEEPING_TIMER, SWEEPING_TIMEOUT);
                    SweepCount++;
                }
                
                if (SweepingDir == CCW)
                {
                    //Sweep in CCW direction
                    UpdateMotors(SweepingCCW[0]);
                    TMR1 = 0;
                    T1CONbits.ON = 1;
                    SweepingDir = CW;          
                }
                else //SwepingDir == CW
                {
                    //Sweep in CW direction
                    UpdateMotors(SweepingCW[0]);
                    TMR1 = 0;
                    T1CONbits.ON = 1;
                    SweepingDir = CCW;
                }         
            }
            break;
            
            case ES_NEW_KEY:
            {
                if (ThisEvent.EventParam == '3')
                {
                    printf("Stop Aligning & Move back to wall\n\r");
                    
                    UpdateMotors(MoveBack2Wall[0]);
                                        
                    TMR1 = 0;
                    T1CONbits.ON = 1;
                }
            }
            break;
        }
    }
    break;
    
    case DoneAligning:
    {
        switch (ThisEvent.EventType)
        {            
            case ES_REACHEDWALL: //Limit switch triggered
            {
                //Stop moving
                UpdateMotors(StopMoving[0]);
                //Disable speed control
                T1CONbits.ON = 0;
                
                printf("ES_REACHEDWALL\n\r");
                
                //Initialize timer for stopping
                ES_Timer_InitTimer(STOPMOVING_TIMER, STOPMOVING_TIMEOUT);
            }
            break;
            
            case ES_TIMEOUT: //Start moving
            {
                //Start moving to Launch
                UpdateMotors(Move2Launch[0]);
                //Zero and start Timer 1 for speed control
                TMR1 = 0;
                T1CONbits.ON = 1;
                    
                //Initialize timer
                ES_Timer_InitTimer(MOVE2LAUNCH_TIMER, MOVE2LAUNCH_TIMEOUT);

                //Set CurrentState to Moving
                CurrentState = Moving;
            }
            break;
        }
    }
    break;
    
    case Moving:
    {
        switch (ThisEvent.EventType)
        {
            case ES_RECEIVEDCOMMAND:
            {
                if (ThisEvent.EventParam == 0) //Query command
                {
                    printf("Received query command\n\r");
                    
                    //Send still moving message
                    Command2SendEvent.EventType = ES_SENDCOMMAND;
                    PostSPIService(Command2SendEvent);
                }
                else if (ThisEvent.EventParam == 0xFF)
                {
                    //Stop moving motors
                    UpdateMotors(StopMoving[0]);
                    T1CONbits.ON = 0;

                    //Post ES_INIT and put us back into Waiting
                    CurrentState = Waiting;
                    ThisEvent.EventType = ES_INIT;
                    PostMotorFSM(ThisEvent);
                }
            }
            break;
            
            case ES_REACHEDWALL:
            {
                printf("Reached Reload position\n\r");
                UpdateMotors(StopMoving[0]);
                
                //Disable speed control
                T1CONbits.ON = 0;
                
                //Send command to leader
                ES_Event_t ReloadCommand;
                ReloadCommand.EventType = ES_REACHEDRELOAD;
                PostSPIService(ReloadCommand);
                
                CurrentState = Waiting;
                ThisEvent.EventType = ES_INIT;
                PostMotorFSM(ThisEvent);
            }
            break;
            
            case ES_TIMEOUT: //Launch position reached
            {
                printf("Reached Launch position\n\r");
                
                UpdateMotors(StopMoving[0]);
                
                //Disable speed control
                T1CONbits.ON = 0;
                
                //Send command to leader
                ES_Event_t LaunchCommand;
                LaunchCommand.EventType = ES_REACHEDLAUNCH;
                PostSPIService(LaunchCommand);

                //Initiate stopping timer
                ES_Timer_InitTimer(SWEEPING_TIMER, STOPMOVING_TIMEOUT);
                SweepingDir = CCW;
                
                //Set CurrentState to Aligning
                CurrentState = Aligning;
            }
            break;
            
            default: 
                ;
            
        }
    }
    break;

    default:
      ;
  }
    // end switch on Current State
  return ReturnEvent;
}

/***************************************************************************
 private functions
 ***************************************************************************/

/* TIMER 1 SETUP */
void ConfigureTimer1WithInterrupt(void)
{
    //Disable Timer 1
    T1CONbits.ON=0;
    //Select the internal PBCLK source
    T1CONbits.TCS=0;
    //Clear timer gate
    T1CONbits.TGATE = 0;
    //Set prescale to 64
    T1CONbits.TCKPS = 0b10;
    //Set period to correspond with 2.5Hz (PR = 15625)
    PR1 = 7812;
    
    //Enable local interrupt
    IEC0SET = _IEC0_T1IE_MASK;
    //Make sure we are set up for multiple interrupt vectors
    INTCONbits.MVEC = 1;
    //Configure interrupt priority to be 6
    IPC1bits.T1IP = 5;
}

/*TIMER 1 SPEED CONTROL ISR*/
void __ISR(_TIMER_1_VECTOR, IPL5SOFT) SpeedControl_ISR(void)
{
    //Disable interrupts globally
    __builtin_disable_interrupts();
  
    //If Timer1 flag is pending
    if (IFS0bits.T1IF == 1)
    {
        //Calculate and set duty cycles for speed control
        PIControlMotor1();
        PIControlMotor2();
        printf("%d, %f, %f\n\r", TargetRPM, CurrentRPM1, CurrentRPM2);
        
        //Clear interrupt
        IFS0CLR = _IFS0_T1IF_MASK;
    }
    //Re-enable interrupts
    __builtin_enable_interrupts();      
}

////////////////////////////////////////////////////////////////////////
/*CONFIGURE TIMER 2 AND INPUT CAPTURE FOR MOTOR SPEEDS*/
void ConfigureTimer2(void)
{
    //Disable Timer 2
    T2CONbits.ON = 0;
    //Select the internal PBCLK source
    T2CONbits.TCS = 0;
    //Clear timer gate
    T2CONbits.TGATE = 0;
    //Set prescale to 16
    T2CONbits.TCKPS = 0b100;
    //Set match count (period) to 0xFFFF
    PR2 = 0xFFFF;
    //Enable local interrupt (rollover)
    IEC0SET = _IEC0_T2IE_MASK;
    //Make sure we are set up for multiple interrupt vectors
    INTCONbits.MVEC = 1;
    //Configure interrupt priority to be 6
    IPC2bits.T2IP = 6;
}

void ConfigureInputCapture1(void)
{
    //Enable input capture
    IC1CONbits.ON = 1;
    //Select 16-bit timer resource capture
    IC1CONbits.C32 = 0;
    //Set timer 2 for input capture
    IC1CONbits.ICTMR = 1;
    //Interrupt on every capture event
    IC1CONbits.ICI = 0;
    //Set mode to detect rising edges only
    IC1CONbits.ICM = 0b011;
    //Enable input capture interrupts
    IEC0SET = _IEC0_IC1IE_MASK;
    //Set input capture interrupt priority to 7
    IPC1bits.IC1IP = 7;
    
    //Configure Pin RB2 as digital input
    PortSetup_ConfigurePullDowns(_Port_B, _Pin_2);
    //Map IC1 to Pin RB2
    IC1R = IC1_PIN;
}

void ConfigureInputCapture2(void)
{
    //Enable input capture
    IC2CONbits.ON = 1;
    //Select 16-bit timer resource capture
    IC2CONbits.C32 = 0;
    //Set timer 2 for input capture
    IC2CONbits.ICTMR = 1;
    //Interrupt on every capture event
    IC2CONbits.ICI = 0;
    //Set mode to detect rising edges only
    IC2CONbits.ICM = 0b011;
    //Enable input capture interrupts
    IEC0SET = _IEC0_IC2IE_MASK;
    //Set input capture interrupt priority to 7
    IPC2bits.IC2IP = 7;
    
    //Configure Pin RB9 as digital input
    PortSetup_ConfigurePullDowns(_Port_B, _Pin_9);
    //Map IC2 to Pin RB9
    IC2R = IC2_PIN;
}

/*INPUT CAPTURE ISRs*/
void __ISR(_TIMER_2_VECTOR, IPL6SOFT) RolloverTimer(void)
{
    //Disable interrupts globally
    __builtin_disable_interrupts();
    
    //IF T2IF pending
    if (IFS0bits.T2IF == 1)
    {
        //Increment RolloverCount
        ++Time1.MyWord.RolloverCount1;
        ++Time2.MyWord.RolloverCount2;
        
        //Clear rollover interrupt
        IFS0CLR = _IFS0_T2IF_MASK;
    }
    
    //Re-enable interrupts
    __builtin_enable_interrupts();
}

void __ISR(_INPUT_CAPTURE_1_VECTOR, IPL7SOFT) CalculateSpeed_Motor1(void)
{
     //Zero Timer4 because motor is not stalled
        TMR4 = 0;
        
        do{            
        //Read IC1BUF into CapturedTime
        Time1.MyWord.CapturedTime1 = (uint16_t)IC1BUF;
        
        //Calculate pulse width by subtracting LastTime from CurrentTime
        PeriodTicks1 = Time1.CurrentTime1 - LastTime1;
        //PeriodTime1 = PeriodTicks1*IC_PRESCALE*1000/PBCLOCK_FREQ;
        
        //Update LastTime to be CurrentTime
        LastTime1 = Time1.CurrentTime1;
    }while(IC1CONbits.ICBNE != 0); //Until the input capture buffer is empty
    
    CurrentRPM1 = PBCLOCK_FREQ*MIN_CONV/((float)PeriodTicks1*IC_PRESCALE*GEAR_RATIO*ENC_PULSE_PER_REV);
    //printf("CurrentRPM1 = %f\n\r",CurrentRPM1);
    
    //Clear IC interrupt
    IFS0CLR = _IFS0_IC1IF_MASK;
    
    //If Timer2 has an interrupt flag pending
    if (IFS0bits.T2IF == 1 && Time1.MyWord.CapturedTime1 < 0x800)
    {
        //Increment RolloverCounter
        ++Time1.MyWord.RolloverCount1;
        ++Time2.MyWord.RolloverCount2;
        
        //Clear the rollover interrupt
        IFS0CLR = _IFS0_T2IF_MASK;
    }
}

void __ISR(_INPUT_CAPTURE_2_VECTOR, IPL7SOFT) CalculateSpeed_Motor2(void)
{
    TMR5 = 0;

    do{       
        //Read IC2BUF into CapturedTime
        Time2.MyWord.CapturedTime2 = (uint16_t)IC2BUF;
        
        //Calculate pulse width by subtracting LastTime from CurrentTime
        PeriodTicks2 = Time2.CurrentTime2 - LastTime2;
        //printf("%d\n\r", PeriodTicks2);

        PeriodTime2 = PeriodTicks2*IC_PRESCALE*1000/PBCLOCK_FREQ;
        
        //Update LastTime to be CurrentTime
        LastTime2 = Time2.CurrentTime2;
    }while(IC2CONbits.ICBNE != 0); //Until the input capture buffer is empty
    
    //Calculate motor speed
    CurrentRPM2 = PBCLOCK_FREQ*MIN_CONV/((float)PeriodTicks2*IC_PRESCALE*GEAR_RATIO*ENC_PULSE_PER_REV);

    //Clear IC interrupt
    IFS0CLR = _IFS0_IC2IF_MASK;
    
    //If Timer2 has an interrupt flag pending
    if (IFS0bits.T2IF == 1 && Time2.MyWord.CapturedTime2 < 0x800)
    {
        //Increment RolloverCounter
        ++Time1.MyWord.RolloverCount1;
        ++Time2.MyWord.RolloverCount2;
        
        //Clear the rollover interrupt
        IFS0CLR = _IFS0_T2IF_MASK;
    }
}

///////////////////////////////////////////////////////////////////////

/*CONFIGURE TIMER 3 AND OUTPUT CAPTURE FOR PWM MOTOR CONTROL*/
void ConfigureTimer3(void)
{
    // Disable Timer 3
    T3CONbits.ON = 0;
    // Select internal PBCLK source
    T3CONbits.TCS = 0;
    // Clear timer gate
    T3CONbits.TGATE = 0;
    // Set prescale to 8
    T3CONbits.TCKPS = 0b011;
    // Set period
    PR3 = PWM_PR;
}

void ConfigureOC1_Motor1(void)
{
    //Configure pin for direction toggle
    PortSetup_ConfigureDigitalOutputs(MOTOR_PORT, MOTOR1_TOGGLEPIN);
    
    //Configure pin for output PWM
    PortSetup_ConfigureDigitalOutputs(MOTOR_PORT, _Pin_4);
    // Map Pin RB4 to OC1
    RPB4R = 0b0101;
    // Disable OC 
    OC1CONbits.ON = 0;
    // Select Timer3 as source
    OC1CONbits.OCTSEL = 1;
    // Set PWM mode with no fault detect
    OC1CONbits.OCM = 0b110;
    // Set initial duty cycle to 0
    OC1RS = 0;
    OC1R = 0;
    LATB = LATB & MOTOR1_BITS; //Set toggle pin high initially
    // Enable OC system
    OC1CONbits.ON = 1;
}

void ConfigureOC2_Motor2(void)
{
    //Configure pin for direction toggle
    PortSetup_ConfigureDigitalOutputs(MOTOR_PORT, MOTOR2_TOGGLEPIN);
    
    //Configure pin for output PWM
    PortSetup_ConfigureDigitalOutputs(MOTOR_PORT, _Pin_11);

    // Map Pin RB11 to OC2
    RPB11R = 0b0101;
    // Disable OC 
    OC2CONbits.ON = 0;
    // Select Timer3 as source
    OC2CONbits.OCTSEL = 1;
    // Set PWM mode with no fault detect
    OC2CONbits.OCM = 0b110;
    // Set initial duty cycle to 0
    OC2RS = 0;
    OC2R = 0;
    LATB = LATB & MOTOR2_BITS; //Set toggle pin high initially
    // Enable OC system
    OC2CONbits.ON = 1;
}

//////////////////////////////////////////////////////////////////

/*UPDATE MOTOR FUNCTION*/
void UpdateMotors(Execution_t ThisCommand)
{
    
    TargetRPM = ThisCommand.TargetSpeed;
    
    //Update motor 1
    if (ThisCommand.Motor1Dir == 0) //If forward direction
    {
        //Keep toggle pin low
        LATB = LATB & ~MOTOR1_BITS; 
        Direction1 = 1;
    }
    else //Reverse direction
    {
        //Switch toggle pin high
        LATB = LATB | MOTOR1_BITS;
        Direction1 = 0;
    }
    OC1RS = ThisCommand.Motor1DC;     //Update motor 1 DC
    
    //Update motor 2
    if (ThisCommand.Motor2Dir == 0) //If forward direction
    {
        //Keep toggle pin low
        LATB = LATB & ~MOTOR2_BITS; 
        Direction2 = 1;
    }
    else //Reverse direction
    {
        //Switch toggle pin high
        LATB = LATB | MOTOR2_BITS;
        Direction2 = 0;
    }
    OC2RS = ThisCommand.Motor2DC;   //Update motor 2 DC
}

///////////////////////////////////////////////////////////////////////////
void PIControlMotor1(void)
{
    static float RPMError1;
    static float SumError1;
    static float RequestedDC1;
    
    RPMError1 = (float)TargetRPM - CurrentRPM1;
    SumError1 += RPMError1;
    RequestedDC1 = 60 + Kp*RPMError1 + Ki*SumError1;
    
    if (RequestedDC1 > PWM_PR) //100% DC
    {
        RequestedDC1 = PWM_PR;
        SumError1 -= RPMError1;
    }
    else if (RequestedDC1 < 0)
    {
        RequestedDC1 = 0;
        SumError1 -= RPMError1;
    }

    //If going forward
    if (Direction1 == 1)
    {
        //Set DC to be Requested DC
        OC1RS = (uint32_t)RequestedDC1;
    }
    else //if reversing
    {
        OC1RS = PWM_PR - (uint32_t)RequestedDC1;
    }
}

//////////////////////////////////////////////////////////////////////////
void PIControlMotor2(void)
{
    static float RPMError2;
    static float SumError2;
    static float RequestedDC2;
    
    RPMError2 = (float)TargetRPM - CurrentRPM2;
    SumError2 += RPMError2;
    RequestedDC2 = 60 + Kp*RPMError2 + Ki*SumError2;
    
    if (RequestedDC2 > PWM_PR) //100% DC
    {
        RequestedDC2 = PWM_PR;
        SumError2 -= RPMError2;
    }
    else if (RequestedDC2 < 0)
    {
        RequestedDC2 = 0;
        SumError2 -= RPMError2;
    }
    //printf("%d, %f\n\r", TargetRPM, CurrentRPM2);

    //If going forward
    if (Direction2 == 1)
    {
        //Set DC to be Requested DC
        OC2RS = (uint32_t)RequestedDC2;
    }
    else //if reversing
    {
        OC2RS = PWM_PR - (uint32_t)RequestedDC2;
    }
}

/////////////////////////////////////////////////////////////////////////
void ConfigureTimer4WithInterrupt(void)
{
    //Disable Timer 4
    T4CONbits.ON=0;
    //Select the internal PBCLK source
    T4CONbits.TCS=0;
    //Clear timer gate
    T4CONbits.TGATE = 0;
    //Set prescale to 256
    T4CONbits.TCKPS = 0b11;
    //Set period to max
    PR4 = 0xFFFF;
    
    //Enable local interrupt
    IEC0SET = _IEC0_T4IE_MASK;
    //Make sure we are set up for multiple interrupt vectors
    INTCONbits.MVEC = 1;
    //Configure interrupt priority to be 4
    IPC4bits.T4IP = 4;
}

void __ISR(_TIMER_4_VECTOR, IPL4SOFT) Motor1Stall(void)
{
    //IF T4IF pending
    if (IFS0bits.T4IF == 1)
    {
        //printf("Motor1Stall\n\r");
        
        //Stop the timer
        T4CONbits.ON = 0;
        
        //Motor is stalled so set current speed to 0
        CurrentRPM1 = 0;
        
        //Clear rollover interrupt
        IFS0CLR = _IFS0_T4IF_MASK;
        
        //Re-enable timer
        T4CONbits.ON = 1;
    }
}

void ConfigureTimer5WithInterrupt(void)
{
    //Disable Timer 5
    T5CONbits.ON=0;
    //Select the internal PBCLK source
    T5CONbits.TCS=0;
    //Clear timer gate
    T5CONbits.TGATE = 0;
    //Set prescale to 256
    T5CONbits.TCKPS = 0b11;
    //Set period to max
    PR5 = 0xFFFF;
    
    //Enable local interrupt
    IEC0SET = _IEC0_T5IE_MASK;
    //Make sure we are set up for multiple interrupt vectors
    INTCONbits.MVEC = 1;
    //Configure interrupt priority to be 4
    IPC5bits.T5IP = 4;
}

void __ISR(_TIMER_5_VECTOR, IPL4SOFT) Motor2Stall(void)
{
    //IF T5IF pending
    if (IFS0bits.T5IF == 1)
    {
        //printf("Motor2Stall\n\r");
        //Stop the timer
        T5CONbits.ON = 0;
        
        //Motor is stalled so set current speed to 0
        CurrentRPM2 = 0;
        
        //Clear rollover interrupt
        IFS0CLR = _IFS0_T5IF_MASK;
        
        //Re-enable timer
        T5CONbits.ON = 1;
    }
}


