/****************************************************************************
 Module
   SPIService.c

 Description
   State machine keeping track of received commands and sending commands
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
#include "ES_Configure.h"
#include "ES_Framework.h"

#include <sys/attribs.h>
#include <stdio.h>
#include <stdint.h>

#include "PIC32_SPI_HAL_KW.h"
#include "SPIService.h"
#include "MotorFSM.h"

/*----------------------------- Module Defines ----------------------------*/

#define KEYSTROKE

#define BRG (10)
#define DEFAULT_COMMAND (3)

#define SS_INPUT (SPI_RPA0)
#define SD_INPUT (SPI_RPA1)
#define SD_OUTPUT (SPI_RPA2)

/*---------------------------- Module Functions ---------------------------*/
void SPI1SetupWithInterrupt(void);
void CommandReceived(void);

/*---------------------------- Module Variables ---------------------------*/
static uint8_t MyPriority;

ES_Event_t CommandEvent;
ES_Event_t MotorEvent;

static uint8_t ReceivedCommand;
static uint8_t Command2Send;


/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitSPIService

 Parameters
     uint8_t : the priority of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, sets up the initial transition and does any
     other required initialization for this state machine
 Notes

 Author
     Kaylie Wong
****************************************************************************/
bool InitSPIService(uint8_t Priority)
{
  ES_Event_t ThisEvent;

  MyPriority = Priority;
  
  //Configure SPI & receive interrupt
  SPI1SetupWithInterrupt();
  
  Command2Send = DEFAULT_COMMAND;
   
  // post the initial transition event
  ThisEvent.EventType = ES_INIT;
  if (ES_PostToService(MyPriority, ThisEvent) == true)
  {
    return true;
  }
  else
  {
    return false;
  }
}

/****************************************************************************
 Function
     PostSPIService

 Parameters
     EF_Event_t ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
    Kaylie Wong
****************************************************************************/
bool PostSPIService(ES_Event_t ThisEvent)
{
  return ES_PostToService(MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunSPIService

 Parameters
   ES_Event_t : the event to process

 Returns
   ES_Event_t, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
    Kaylie Wong
****************************************************************************/
ES_Event_t RunSPIService(ES_Event_t ThisEvent)
{
  ES_Event_t ReturnEvent;
  ReturnEvent.EventType = ES_NO_EVENT; // assume no errors

  switch (ThisEvent.EventType)
  {
    case ES_RECEIVEDCOMMAND:        
    {
        //Post event to Motor Service
        PostMotorFSM(CommandEvent);
        printf("ES_RECEIVEDCOMMAND = %d\n\r", ReceivedCommand);
    }
    break;
    
    case ES_SENDCOMMAND:
    {
        //printf("Send command event received \r\n");       
        SPIOperate_SPI1_Send8Wait(Command2Send);
        printf("Sending command %d\n\r", Command2Send);
        Command2Send = DEFAULT_COMMAND;
    }
    break;
    
    case ES_REACHEDLAUNCH:
    {
        Command2Send = 4;
    }
    break;

    case ES_REACHEDRELOAD:
    {
        Command2Send = 1;
    }
    break;
    
    default:
    ;
        
  }                                   
  return ReturnEvent;
}

/***************************************************************************
 private functions
 ***************************************************************************/
void SPI1SetupWithInterrupt(void)
{
    //Disable SPI receive interrupt in IEC1 register
    IEC1CLR = _IEC1_SPI1RXIE_MASK;

    //Configure SPI1 module
    SPISetup_BasicConfig(SPI_SPI1);

    //Set SPI1 module to Follower mode with Pin RB14 as PBCLK input
    SPISetup_SetFollower(SPI_SPI1);

    //Set SPIBRG
    SPISetup_SetBitTime(SPI_SPI1, BRG);

    //Set active edge to be second edge, CKE=0
    SPISetup_SetActiveEdge(SPI_SPI1, SPI_SECOND_EDGE);
    
    //Set idle state of SPI clock, CKP=1
    SPISetup_SetClockIdleState(SPI_SPI1, SPI_CLK_HI);
    
    //Disable enhanced buffer
    SPISetEnhancedBuffer(SPI_SPI1, false);
  
    //Clear receive buffer
    SPI1BUF = 0;

    //Map pins
    //SS1 -> RA0
    SPISetup_MapSSInput(SPI_SPI1, SS_INPUT); 
    //SS1R = 0;
    //SDO1 -> RA2
    SPISetup_MapSDOutput(SPI_SPI1, SD_OUTPUT);
    //RPA2R = 0b0011;
    //SDI1 -> RA1
    SPISetup_MapSDInput(SPI_SPI1, SD_INPUT);
    //SDI1R = 0;

    //Set 8-bit transfers
    SPISetup_SetXferWidth(SPI_SPI1, SPI_8BIT);

    /*Setup receive interrupt with priority = 7*/
    //Clear SPI1 interrupt flag
    IFS1CLR = _IFS1_SPI1RXIF_MASK;

    //Write SPIx interrupt priority in PICx register
    IPC7bits.SPI1IP = 7;

    //Interrupt when the buffer is not empty
    SPI1CONbits.SRXISEL = 1;

    //Enable receive interrupt
    IEC1SET = _IEC1_SPI1RXIE_MASK;
    
    //Enable SPI1
    SPISetup_EnableSPI(SPI_SPI1);

    printf("SPI Setup Complete\n\r");
}

//////////////////////////////////////////////////////////
void __ISR(_SPI_1_VECTOR, IPL7SOFT) CommandReceived(void)
{
    CommandEvent.EventType = ES_RECEIVEDCOMMAND;
    
    //Read SPI1BUF to ReceivedCommand
    ReceivedCommand = (uint8_t)SPI1BUF;
    
    CommandEvent.EventParam = ReceivedCommand;
    
    //printf("Received Command = %d\n\r", ReceivedCommand);
    
    //Post event to SPI Service
    PostSPIService(CommandEvent);
    
    //Clear interrupt flag
    IFS1CLR = _IFS1_SPI1RXIF_MASK;
}